@@ -52,6 +52,9 @@
 #include "DSSIProgramEvent.h"
 #include "MidiLearnEvent.h"
 #include "DisconnectNodeEvent.h"
+#include "RegisterClientEvent.h"
+#include "UnregisterClientEvent.h"
+
 
 using std::cerr; using std::cout; using std::endl;
 
@@ -78,7 +81,7 @@ OSCReceiver::OSCReceiver(const char* const port)
 	}
 
 	// For debugging, print all incoming OSC messages
-	//lo_server_thread_add_method(m_st, NULL, NULL, generic_cb, NULL);
+	lo_server_thread_add_method(m_st, NULL, NULL, generic_cb, NULL);
 
 	// Commands
 	lo_server_thread_add_method(m_st, "/om/ping", "i", ping_cb, this);
@@ -136,6 +139,50 @@ OSCReceiver::activate()
 }
 
 
+bool
+OSCReceiver::register_client(const string& host, const string& port)
+{
+	bool found = false;
+	for (list<ClientRecord*>::iterator i = m_clients.begin(); i != m_clients.end(); ++i)
+		if ((*i)->hostname() == host && (*i)->port() == port)
+			found = true;
+
+	if (!found) {
+		ClientRecord* cr = new ClientRecord(host, port);
+		m_clients.push_back(cr);
+		cout << "Added client " << host << ":" << port
+			<< " to clients list (" << m_clients.size() << " clients)" << endl;
+		return true;
+	} else {
+		cout << "Client already registered." << endl;
+		return false;
+	}
+}
+
+
+bool
+OSCReceiver::unregister_client(const string& host, const string& port)
+{
+	bool found = false;
+	for (list<ClientRecord*>::iterator i = m_clients.begin(); i != m_clients.end(); ++i) {
+		if ((*i)->hostname() == host && (*i)->port() == port) {
+			found = true;
+			m_clients.erase(i);
+			delete (*i);
+			break;
+		}
+	}
+	
+	if (found) {
+		cout << "Unregistered client.\n";
+		return true;
+	} else {
+		cerr << "ERROR: Unable to find client to unregister!" << endl;
+		return false;
+	}
+}
+
+
 lo_address
 OSCReceiver::get_reply_addr(const lo_message msg)
 {
@@ -214,30 +261,16 @@ OSCReceiver::m_quit_cb(const char* path, const char* types, lo_arg** argv, int a
 int
 OSCReceiver::m_register_client_cb(const char* path, const char* types, lo_arg** argv, int argc, lo_message msg)
 {
-	const int   request_id   =  argv[0]->i;
+	const int request_id   =  argv[0]->i;
 	
 	lo_address incoming_addr = lo_message_get_source(msg);
 	const char* const incoming_host = lo_address_get_hostname(incoming_addr);
 	const char* const incoming_port = lo_address_get_port(incoming_addr);
 	
-	cout << "Registering client " << incoming_host << ":" << incoming_port << endl;
-
-	bool found = false;
-	for (list<ClientRecord*>::iterator i = m_clients.begin(); i != m_clients.end(); ++i)
-		if (!strcmp((*i)->hostname().c_str(), incoming_host)
-				&& !strcmp((*i)->port().c_str(), incoming_port))
-			found = true;
-
-	if (!found) {
-		ClientRecord* cr = new ClientRecord(incoming_host, incoming_port);
-		m_clients.push_back(cr);
-		cout << "Added client " << incoming_host << ":" << incoming_port
-			<< " to clients list (" << m_clients.size() << " clients)" << endl;
-	} else {
-		cout << "Client already registered." << endl;
-	}
-	
-	Request(incoming_addr, request_id).respond_ok();
+	// FIXME: leak!
+	Request r = Request(lo_address_new(incoming_host, incoming_port), request_id);
+	RegisterClientEvent* ev = new RegisterClientEvent(r, incoming_host, incoming_port);
+	om->jack_driver()->push_slow_event(ev);
 	
 	return 0;
 }
@@ -252,6 +285,7 @@ OSCReceiver::m_register_client_cb(const char* path, const char* types, lo_arg**
 int
 OSCReceiver::m_register_client_host_cb(const char* path, const char* types, lo_arg** argv, int argc, lo_message msg)
 {
+	lo_address        addr       = get_reply_addr(msg);
 	const int         request_id =  argv[0]->i;
 	const char* const host       = &argv[1]->s;
 	const int         port       =  argv[2]->i;
@@ -261,30 +295,16 @@ OSCReceiver::m_register_client_host_cb(const char* path, const char* types, lo_a
 	
 	cout << "Registering client " << host << ":" << port_str << endl;
 
-	bool found = false;
-	for (list<ClientRecord*>::iterator i = m_clients.begin(); i != m_clients.end(); ++i)
-		if (!strcmp((*i)->hostname().c_str(), host)
-				&& !strcmp((*i)->port().c_str(), port_str))
-			found = true;
-
-	if (!found) {
-		ClientRecord* cr = new ClientRecord(host, port_str);
-		m_clients.push_back(cr);
-		cout << "Added client " << host << ":" << port_str
-			<< " to clients list (" << m_clients.size() << " clients)" << endl;
-	} else {
-		cout << "Client already registered." << endl;
-	}
+	RegisterClientEvent* ev = new RegisterClientEvent(Request(addr, request_id), host, port_str);
 	
-	lo_address incoming_addr = lo_message_get_source(msg);
-	Request(incoming_addr, request_id).respond_ok();
+	om->jack_driver()->push_slow_event(ev);
 	
 	return 0;
 }
 
 
 /** \page engine_osc_namespace
- * <p> \b /om/engine/unregister_client - Unregisters a client so it will no longer receive messages from the engine.
+ * <p> \b /om/engine/unregister_client - Unregisters a client (using source address)
  * \arg \b request-id (integer) </p> \n \n
  */
 int
@@ -301,24 +321,41 @@ OSCReceiver::m_unregister_client_cb(const char* path, const char* types, lo_arg*
 	const char* const incoming_host = lo_address_get_hostname(addr);
 	const char* const incoming_port = lo_address_get_port(addr);
 	
-	bool found = false;
-	for (list<ClientRecord*>::iterator i = m_clients.begin(); i != m_clients.end(); ++i) {
-		if (!strcmp((*i)->hostname().c_str(), incoming_host)
-				&& !strcmp((*i)->port().c_str(), incoming_port)) {
-			found = true;
-			Request(addr, request_id).respond_ok();
-			m_clients.erase(i);
-			delete (*i);
-			break;
-		}
+	UnregisterClientEvent* ev = new UnregisterClientEvent(
+		Request(addr, request_id), incoming_host, incoming_port);
+	
+	om->jack_driver()->push_slow_event(ev);
+	
+	return 0;
+}
+
+
+/** \page engine_osc_namespace
+ * <p> \b /om/engine/unregister_client - Unregisters a client (using specified host/port)
+ * \arg \b request-id (integer)
+ * \arg \b host (string) - Hostname of client to unregister
+ * \arg \b port (integer) - Port of client to unregister </p> \n \n
+ */
+int
+OSCReceiver::m_unregister_client_host_cb(const char* path, const char* types, lo_arg** argv, int argc, lo_message msg)
+{
+	lo_address        addr       = get_reply_addr(msg);
+	const int         request_id =  argv[0]->i;
+	const char* const host       = &argv[1]->s;
+	const int         port       =  argv[2]->i;
+	
+	char port_str[8];
+	snprintf(port_str, 8, "%d", port);
+	
+	if (addr == NULL) {
+		cerr << "WARNING: Unregistration request from unregistered client." << endl;
+		return 0;
 	}
 	
-	if (found) {
-		cout << "Unregistered client.\n";
-	} else {
-		cerr << "ERROR: Unable to find client to unregister!" << std::endl;
-		Request(addr, request_id).respond_error("Unable to find client to unregister.");
-	}
+	UnregisterClientEvent* ev = new UnregisterClientEvent(
+		Request(addr, request_id), host, port_str);
+	
+	om->jack_driver()->push_slow_event(ev);
 	
 	return 0;
 }
