@@ -80,6 +80,9 @@ public:
 
 	void activate();
 
+	bool register_client(const string& host, const string& port);
+	bool unregister_client(const string& host, const string& port);
+	
 	const list<ClientRecord*>& clients() const { return m_clients; }
 
 private:
@@ -95,69 +98,71 @@ private:
 	static int  unknown_cb(const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
 
 	// Static callbacks, these call the non-static member functions below
-	inline static int quit_cb                 (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
-	inline static int ping_cb                 (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
-	inline static int register_client_cb      (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
-	inline static int register_client_host_cb (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
-	inline static int unregister_client_cb    (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
-	inline static int load_plugins_cb         (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
-	inline static int send_plugins_cb         (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
-	inline static int send_all_objects_cb     (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
-	inline static int engine_activate_cb      (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
-	inline static int engine_deactivate_cb    (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
-	inline static int engine_enable_cb        (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
-	inline static int engine_disable_cb       (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
-	inline static int create_patch_cb         (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
-	inline static int destroy_patch_cb        (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
-	inline static int patch_add_node_cb       (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
-	inline static int enable_patch_cb         (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
-	inline static int disable_patch_cb        (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
-	inline static int patch_remove_node_cb    (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
-	inline static int patch_connect_cb        (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
-	inline static int patch_disconnect_cb     (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
-	inline static int patch_disconnect_all_cb (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
-	inline static int patch_request_control_cb(const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
-	inline static int set_port_value_cb       (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
-	inline static int set_port_value_voice_cb (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
-	inline static int set_port_value_slow_cb  (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
-	inline static int dssi_cb                 (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
-	inline static int note_on_cb              (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
-	inline static int note_off_cb             (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
-	inline static int midi_learn_cb           (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
-	inline static int metadata_get_cb         (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
-	inline static int metadata_set_cb         (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
+	inline static int quit_cb                  (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
+	inline static int ping_cb                  (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
+	inline static int register_client_cb       (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
+	inline static int register_client_host_cb  (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
+	inline static int unregister_client_cb     (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
+	inline static int unregister_client_host_cb(const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
+	inline static int load_plugins_cb          (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
+	inline static int send_plugins_cb          (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
+	inline static int send_all_objects_cb      (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
+	inline static int engine_activate_cb       (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
+	inline static int engine_deactivate_cb     (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
+	inline static int engine_enable_cb         (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
+	inline static int engine_disable_cb        (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
+	inline static int create_patch_cb          (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
+	inline static int destroy_patch_cb         (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
+	inline static int patch_add_node_cb        (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
+	inline static int enable_patch_cb          (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
+	inline static int disable_patch_cb         (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
+	inline static int patch_remove_node_cb     (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
+	inline static int patch_connect_cb         (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
+	inline static int patch_disconnect_cb      (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
+	inline static int patch_disconnect_all_cb  (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
+	inline static int patch_request_control_cb (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
+	inline static int set_port_value_cb        (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
+	inline static int set_port_value_voice_cb  (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
+	inline static int set_port_value_slow_cb   (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
+	inline static int dssi_cb                  (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
+	inline static int note_on_cb               (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
+	inline static int note_off_cb              (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
+	inline static int midi_learn_cb            (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
+	inline static int metadata_get_cb          (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
+	inline static int metadata_set_cb          (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver);
 
-	int m_quit_cb                 (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
-	int m_ping_cb                 (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
-	int m_register_client_cb      (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
-	int m_register_client_host_cb (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
-	int m_unregister_client_cb    (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
-	int m_load_plugins_cb         (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
-	int m_send_plugins_cb         (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
-	int m_send_all_objects_cb     (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
-	int m_engine_activate_cb      (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
-	int m_engine_deactivate_cb    (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
-	int m_engine_enable_cb        (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
-	int m_engine_disable_cb       (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
-	int m_create_patch_cb         (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
-	int m_destroy_patch_cb        (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
-	int m_patch_add_node_cb       (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
-	int m_enable_patch_cb         (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
-	int m_disable_patch_cb        (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
-	int m_patch_remove_node_cb    (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
-	int m_patch_connect_cb        (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
-	int m_patch_disconnect_cb     (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
-	int m_patch_disconnect_all_cb (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
-	int m_patch_request_control_cb(const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
-	int m_set_port_value_cb       (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
-	int m_set_port_value_voice_cb (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
-	int m_set_port_value_slow_cb  (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
-	int m_dssi_cb                 (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
-	int m_note_on_cb              (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
-	int m_note_off_cb             (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
-	int m_midi_learn_cb           (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
-	int m_metadata_get_cb         (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
-	int m_metadata_set_cb         (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
+	int m_quit_cb                  (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
+	int m_ping_cb                  (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
+	int m_register_client_cb       (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
+	int m_register_client_host_cb  (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
+	int m_unregister_client_cb     (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
+	int m_unregister_client_host_cb(const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
+	int m_load_plugins_cb          (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
+	int m_send_plugins_cb          (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
+	int m_send_all_objects_cb      (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
+	int m_engine_activate_cb       (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
+	int m_engine_deactivate_cb     (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
+	int m_engine_enable_cb         (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
+	int m_engine_disable_cb        (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
+	int m_create_patch_cb          (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
+	int m_destroy_patch_cb         (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
+	int m_patch_add_node_cb        (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
+	int m_enable_patch_cb          (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
+	int m_disable_patch_cb         (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
+	int m_patch_remove_node_cb     (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
+	int m_patch_connect_cb         (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
+	int m_patch_disconnect_cb      (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
+	int m_patch_disconnect_all_cb  (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
+	int m_patch_request_control_cb (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
+	int m_set_port_value_cb        (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
+	int m_set_port_value_voice_cb  (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
+	int m_set_port_value_slow_cb   (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
+	int m_dssi_cb                  (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
+	int m_note_on_cb               (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
+	int m_note_off_cb              (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
+	int m_midi_learn_cb            (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
+	int m_metadata_get_cb          (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
+	int m_metadata_set_cb          (const char* path, const char* types, lo_arg** argv, int argc, lo_message msg);
 };
 
 
@@ -189,6 +194,10 @@ inline int OSCReceiver::unregister_client_cb(const char* path, const char* types
 	return ((OSCReceiver*)osc_receiver)->m_unregister_client_cb(path, types, argv, argc, msg);
 }
 
+inline int OSCReceiver::unregister_client_host_cb(const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver) {
+	return ((OSCReceiver*)osc_receiver)->m_unregister_client_host_cb(path, types, argv, argc, msg);
+}
+
 inline int OSCReceiver::load_plugins_cb(const char* path, const char* types, lo_arg** argv, int argc, lo_message msg, void* osc_receiver) {
 	return ((OSCReceiver*)osc_receiver)->m_load_plugins_cb(path, types, argv, argc, msg);
 }
